@@ -1,14 +1,22 @@
-import collections
+from __future__ import annotations
+
 import logging
 import math
-from typing import Iterable
+from collections import defaultdict, deque
+from collections.abc import Iterable
+from datetime import timedelta
+from typing import TYPE_CHECKING, cast
 
 import tlz as toolz
 from tornado.ioloop import IOLoop, PeriodicCallback
 
 from dask.utils import parse_timedelta
 
-from ..metrics import time
+from distributed.metrics import time
+
+if TYPE_CHECKING:
+    from distributed.scheduler import WorkerState
+
 
 logger = logging.getLogger(__name__)
 
@@ -60,7 +68,7 @@ class AdaptiveCore:
     ----------
     minimum: int
         The minimum number of allowed workers
-    maximum: int
+    maximum: int | inf
         The maximum number of allowed workers
     wait_count: int
         The number of scale-down requests we should receive before actually
@@ -69,17 +77,32 @@ class AdaptiveCore:
         The amount of time, like ``"1s"`` between checks
     """
 
+    minimum: int
+    maximum: int | float
+    wait_count: int
+    interval: int | float
+    periodic_callback: PeriodicCallback | None
+    plan: set[WorkerState]
+    requested: set[WorkerState]
+    observed: set[WorkerState]
+    close_counts: defaultdict[WorkerState, int]
+    _adapting: bool
+    log: deque[tuple[float, dict]]
+
     def __init__(
         self,
         minimum: int = 0,
-        maximum: int = math.inf,
+        maximum: int | float = math.inf,
         wait_count: int = 3,
-        interval: str = "1s",
+        interval: str | int | float | timedelta = "1s",
     ):
+        if not isinstance(maximum, int) and not math.isinf(maximum):
+            raise TypeError(f"maximum must be int or inf; got {maximum}")
+
         self.minimum = minimum
         self.maximum = maximum
         self.wait_count = wait_count
-        self.interval = parse_timedelta(interval, "seconds") if interval else interval
+        self.interval = parse_timedelta(interval, "seconds")
         self.periodic_callback = None
 
         def f():
@@ -99,10 +122,7 @@ class AdaptiveCore:
                     await core.adapt()
 
             self.periodic_callback = PeriodicCallback(_adapt, self.interval * 1000)
-            try:
-                self.loop.add_callback(f)
-            except AttributeError:
-                IOLoop.current().add_callback(f)
+            self.loop.add_callback(f)
 
         try:
             self.plan = set()
@@ -112,11 +132,11 @@ class AdaptiveCore:
             pass
 
         # internal state
-        self.close_counts = collections.defaultdict(int)
+        self.close_counts = defaultdict(int)
         self._adapting = False
-        self.log = collections.deque(maxlen=10000)
+        self.log = deque(maxlen=10000)
 
-    def stop(self):
+    def stop(self) -> None:
         logger.info("Adaptive stop")
 
         if self.periodic_callback:
@@ -138,17 +158,17 @@ class AdaptiveCore:
         """Used internally, like target, but respects minimum/maximum"""
         n = await self.target()
         if n > self.maximum:
-            n = self.maximum
+            n = cast(int, self.maximum)
 
         if n < self.minimum:
             n = self.minimum
 
         return n
 
-    async def scale_down(self, n: int):
+    async def scale_down(self, n: int) -> None:
         raise NotImplementedError()
 
-    async def scale_up(self, workers: Iterable):
+    async def scale_up(self, workers: Iterable) -> None:
         raise NotImplementedError()
 
     async def recommendations(self, target: int) -> dict:
@@ -163,34 +183,34 @@ class AdaptiveCore:
             self.close_counts.clear()
             return {"status": "same"}
 
-        elif target > len(plan):
+        if target > len(plan):
             self.close_counts.clear()
             return {"status": "up", "n": target}
 
-        elif target < len(plan):
-            not_yet_arrived = requested - observed
-            to_close = set()
-            if not_yet_arrived:
-                to_close.update(toolz.take(len(plan) - target, not_yet_arrived))
+        # target < len(plan)
+        not_yet_arrived = requested - observed
+        to_close = set()
+        if not_yet_arrived:
+            to_close.update(toolz.take(len(plan) - target, not_yet_arrived))
 
-            if target < len(plan) - len(to_close):
-                L = await self.workers_to_close(target=target)
-                to_close.update(L)
+        if target < len(plan) - len(to_close):
+            L = await self.workers_to_close(target=target)
+            to_close.update(L)
 
-            firmly_close = set()
-            for w in to_close:
-                self.close_counts[w] += 1
-                if self.close_counts[w] >= self.wait_count:
-                    firmly_close.add(w)
+        firmly_close = set()
+        for w in to_close:
+            self.close_counts[w] += 1
+            if self.close_counts[w] >= self.wait_count:
+                firmly_close.add(w)
 
-            for k in list(self.close_counts):  # clear out unseen keys
-                if k in firmly_close or k not in to_close:
-                    del self.close_counts[k]
+        for k in list(self.close_counts):  # clear out unseen keys
+            if k in firmly_close or k not in to_close:
+                del self.close_counts[k]
 
-            if firmly_close:
-                return {"status": "down", "workers": list(firmly_close)}
-            else:
-                return {"status": "same"}
+        if firmly_close:
+            return {"status": "down", "workers": list(firmly_close)}
+        else:
+            return {"status": "same"}
 
     async def adapt(self) -> None:
         """
@@ -231,3 +251,7 @@ class AdaptiveCore:
 
     def __del__(self):
         self.stop()
+
+    @property
+    def loop(self) -> IOLoop:
+        return IOLoop.current()
