@@ -27,28 +27,26 @@ class PublishExtension:
 
         self.scheduler.handlers.update(handlers)
 
+    @log_errors
     def put(self, keys=None, data=None, name=None, override=False, client=None):
-        with log_errors():
-            if not override and name in self.datasets:
-                raise KeyError("Dataset %s already exists" % name)
-            self.scheduler.client_desires_keys(keys, f"published-{stringify(name)}")
-            self.datasets[name] = {"data": data, "keys": keys}
-            return {"status": "OK", "name": name}
+        if not override and name in self.datasets:
+            raise KeyError("Dataset %s already exists" % name)
+        self.scheduler.client_desires_keys(keys, f"published-{stringify(name)}")
+        self.datasets[name] = {"data": data, "keys": keys}
+        return {"status": "OK", "name": name}
 
+    @log_errors
     def delete(self, name=None):
-        with log_errors():
-            out = self.datasets.pop(name, {"keys": []})
-            self.scheduler.client_releases_keys(
-                out["keys"], f"published-{stringify(name)}"
-            )
+        out = self.datasets.pop(name, {"keys": []})
+        self.scheduler.client_releases_keys(out["keys"], f"published-{stringify(name)}")
 
+    @log_errors
     def list(self, *args):
-        with log_errors():
-            return list(sorted(self.datasets.keys(), key=str))
+        return list(sorted(self.datasets.keys(), key=str))
 
+    @log_errors
     def get(self, name=None, client=None):
-        with log_errors():
-            return self.datasets.get(name, None)
+        return self.datasets.get(name, None)
 
 
 class Datasets(MutableMapping):
